#!/usr/bin/python
#
# Software License Agreement (BSD License)
#
# Copyright (c) 2008, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of the Willow Garage nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

##\author Eric Berger, Kevin Watts

##\brief Converts diagnostics log files into CSV's for analysis

PKG = 'diagnostic_analysis'
import roslib; roslib.load_manifest(PKG)
import diagnostic_msgs.msg
import time, sys, os
import operator, tempfile, subprocess

from optparse import OptionParser

from diagnostic_analysis.exporter import LogExporter

if __name__ == '__main__':
    # Allow user to set output directory
    parser = OptionParser()
    parser.add_option("-d", "--directory", dest="directory",
                      help="Write output to DIR/output. Default: %s" % PKG, metavar="DIR",
                      default=roslib.packages.get_pkg_dir(PKG), action="store")
    options, args = parser.parse_args()

    exporters = []

    print('Output directory: %s/output' % options.directory)

    try:
        for i, f in enumerate(args):
            filepath = 'output/%s_csv' % os.path.basename(f)[0:os.path.basename(f).find('.')]
            
            output_dir = os.path.join(options.directory,  filepath)
            print("Processing file %s. File %d of %d." % (os.path.basename(f), i + 1, len(args)))

            exp = LogExporter(output_dir, f)
            exp.process_log()
            exp.finish_logfile()
            exporters.append(exp)

        print('Finished processing files.')
    except:
        import traceback
        print("Caught exception processing log file")
        traceback.print_exc()
